/*
 * ImageFile.swift
 * SlidesPro
 *
 * Created by Gabriel Miro on 30/3/16.
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import Cocoa

class ImageFile {
  
  private(set) var thumbnail: NSImage?
  private(set) var fileName: String
  private(set) var url: NSURL
  
  init (url: NSURL) {
    self.url = url
    if let name = url.lastPathComponent {
      fileName = name
    } else {
      fileName = ""
    }
    let imageSource = CGImageSourceCreateWithURL(url.absoluteURL, nil)
    if let imageSource = imageSource {
      guard CGImageSourceGetType(imageSource) != nil else { return }
      thumbnail = getThumbnailImage(imageSource)
    }
  }
  
  private func getThumbnailImage(imageSource: CGImageSource) -> NSImage? {
    let thumbnailOptions = [
      String(kCGImageSourceCreateThumbnailFromImageIfAbsent): true,
      String(kCGImageSourceThumbnailMaxPixelSize): 160
    ]
    guard let thumbnailRef = CGImageSourceCreateThumbnailAtIndex(imageSource, 0, thumbnailOptions) else { return nil}
    return NSImage(CGImage: thumbnailRef, size: NSSize.zero)
  }
  
}
